/*-
 * Copyright (c) 2018 UPLEX Nils Goroll Systemoptimierung
 * All rights reserved
 *
 * Author: Geoffrey Simmons <geoffrey.simmons@uplex.de>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef _LOG_H_INCLUDED
#define _LOG_H_INCLUDED

#include <stdio.h>
#include <vapi/vsl.h>

int dispatch(struct VSLQ *vslq, int n);

// Necessary because the field name is a go keyword.
static enum VSL_transaction_e
txtype(struct VSL_transaction *tx)
{
	return tx->type;
}

// The rest are necessary due to the use of macros.
static inline int
slt_max()
{
	return SLT__MAX;
}

static inline unsigned
vxid(struct VSL_cursor *c)
{
	return (VSL_ID(c->rec.ptr));
}

static inline enum VSL_tag_e
tag(struct VSL_cursor *c)
{
	return (VSL_TAG(c->rec.ptr));
}

static inline const char *
cdata(struct VSL_cursor *c)
{
	return (VSL_CDATA(c->rec.ptr));
}

static inline int
len(struct VSL_cursor *c)
{
        // Subtracting one because Go doesn't need the terminating null.
	return (VSL_LEN(c->rec.ptr) - 1);
}

static inline int
client(struct VSL_cursor *c)
{
	return (VSL_CLIENT(c->rec.ptr));
}

static inline int
backend(struct VSL_cursor *c)
{
	return (VSL_BACKEND(c->rec.ptr));
}

static inline int
binary(enum VSL_tag_e tag)
{
	return (VSL_tagflags[tag] & SLT_F_BINARY);
}

static inline int
unsafe(enum VSL_tag_e tag)
{
	return (VSL_tagflags[tag] & SLT_F_UNSAFE);
}

#endif
